@@ -87,7 +87,7 @@ merge_measures <- function(...) {
 #'
 #' @export
 to_global <- function(...) {
-  .Defunct("measures_global", package = "sentometrics")
+  .Defunct("aggregate.sento_measures", package = "sentometrics")
 }
 
 #' @rdname sentometrics-defunct
@@ -234,3 +234,21 @@ measures_merge <- function(...) {
   .Defunct("aggregate.sento_measures", package = "sentometrics")
 }
 
+#' @rdname sentometrics-defunct
+#'
+#' @param ... allowed input arguments.
+#'
+#' @export
+measures_global <- function(...) {
+  .Defunct("aggregate.sento_measures", package = "sentometrics")
+}
+
+#' @rdname sentometrics-defunct
+#'
+#' @param ... allowed input arguments.
+#'
+#' @export
+sento_app <- function(...) {
+  stop("Use the sento_app() function now put into the sentometrics.app package.")
+}
+
